@@ -1,10 +1,12 @@
-from .iam_pomes import (
-    IamServer,
-    login_callback, token_exchange,
-    user_login, user_logout, user_token
+from .iam_actions import (
+    action_callback, action_exchange,
+    action_login, action_logout, action_token
+)
+from .iam_common import (
+    IamServer
 )
 from .iam_services import (
-     logger_register
+     jwt_required, logger_register
 )
 from .jusbr_pomes import (
     jusbr_setup, jusbr_get_token
@@ -20,12 +22,13 @@ from .token_pomes import (
 )
 
 __all__ = [
-    # iam_pomes
+    # iam_actions
+    "action_callback", "action_exchange",
+    "action_login", "action_logout", "action_token",
+    # iam_commons
     "IamServer",
-    "login_callback", "token_exchange",
-    "user_login", "user_logout", "user_token",
     # iam_services
-    "logger_register",
+    "jwt_required", "logger_register",
     # jusbr_pomes
     "jusbr_setup", "jusbr_get_token",
     # keycloak_pomes
