@@ -50,23 +50,108 @@ class IamServer(StrEnum):
 # }
 _IAM_SERVERS: Final[dict[IamServer, dict[str, Any]]] = {}
 
-# the lock protecting the data in '_IAM_SERVER'
+# the lock protecting the data in '_IAM_SERVERS'
 # (because it is 'Final' and set at declaration time, it can be accessed through simple imports)
 _iam_lock: Final[RLock] = RLock()
 
 
+def _iam_server_from_endpoint(endpoint: str,
+                              errors: list[str] | None,
+                              logger: Logger | None) -> IamServer | None:
+    """
+    Retrieve the registered *IAM* server associated with the service's invocation *endpoint*.
+
+    :param endpoint: the service's invocation endpoint
+    :param errors: incidental error messages
+    :param logger: optional logger
+    :return: the corresponding *IAM* server, or *None* if one could not be obtained
+    """
+    # declare the return variable
+    result: IamServer | None
+
+    if endpoint.startswith("jusbr"):
+        result = IamServer.IAM_JUSRBR
+    elif endpoint.startswith("keycloak"):
+        result = IamServer.IAM_KEYCLOAK
+    else:
+        result = None
+        msg: str = f"Unable to find a IAM server to service endpoint '{endpoint}'"
+        if logger:
+            logger.error(msg=msg)
+        if isinstance(errors, list):
+            errors.append(msg)
+
+    return result
+
+
+def _iam_server_from_issuer(issuer: str,
+                            errors: list[str] | None,
+                            logger: Logger | None) -> IamServer | None:
+    """
+    Retrieve the registered *IAM* server associated with the token's *issuer*.
+
+    :param issuer: the token's issuer
+    :param errors: incidental error messages
+    :param logger: optional logger
+    :return: the corresponding *IAM* server, or *None* if one could not be obtained
+    """
+    # initialize the return variable
+    result: IamServer | None = None
+
+    for iam_server, server_data in _IAM_SERVERS.items():
+        if server_data["base-url"] == issuer:
+            result = IamServer(iam_server)
+            break
+
+    if not result:
+        msg: str = f"Unable to find a IAM server associated with token issuer '{issuer}'"
+        if logger:
+            logger.error(msg=msg)
+        if isinstance(errors, list):
+            errors.append(msg)
+
+    return result
+
+
 def _get_public_key(iam_server: IamServer,
                     errors: list[str] | None,
                     logger: Logger | None) -> str:
     """
     Obtain the public key used by *iam_server* to sign the authentication tokens.
 
-    The public key is saved in *iam_server*'s registry.
+    This is accomplished by requesting the token issuer for its *JWKS* (JSON Web Key Set),
+    containing the public keys used for various purposes, as indicated in the attribute *use*:
+        - *enc*: the key is intended for encryption
+        - *sig*: the key is intended for digital signature
+        - *wrap*: the key is intended for key wrapping
+
+    A typical JWKS set has the following format (for simplicity, 'n' and 'x5c' are truncated):
+        {
+            "keys": [
+                {
+                    "kid": "X2QEcSQ4Tg2M2EK6s2nhRHZH_GwD_zxZtiWVwP4S0tg",
+                    "kty": "RSA",
+                    "alg": "RSA256",
+                    "use": "sig",
+                    "n": "tQmDmyM3tMFt5FMVMbqbQYpaDPf6A5l4e_kTVDBiHrK_bRlGfkk8hYm5SNzNzCZ...",
+                    "e": "AQAB",
+                    "x5c": [
+                        "MIIClzCCAX8CBgGZY0bqrTANBgkqhkiG9w0BAQsFADAPMQ0wCwYDVQQDDARpanVk..."
+                    ],
+                    "x5t": "MHfVp4kBjEZuYOtiaaGsfLCL15Q",
+                    "x5t#S256": "QADezSLgD8emuonBz8hn8ghTnxo7AHX4NVNkr4luEhk"
+                },
+                ...
+            ]
+        }
+
+    Once the signature key is obtained, it is converted from its original *JWK* (JSON Web Key) format
+    to *PEM* (Privacy-Enhanced Mail) format. The public key is saved in *iam_server*'s registry.
 
     :param iam_server: the reference registered *IAM* server
     :param errors: incidental error messages
     :param logger: optional logger
-    :return: the public key in *PEM* format, or *None* if the server is unknown
+    :return: the public key in *PEM* format, or *None* if error
     """
     # initialize the return variable
     result: str | None = None
@@ -77,9 +162,10 @@ def _get_public_key(iam_server: IamServer,
     if registry:
         now: int = int(datetime.now(tz=TZ_LOCAL).timestamp())
         if now > registry["pk-expiration"]:
-            # obtain a new public key
+            # obtain the JWKS (JSON Web Key Set) from the token issuer
             url: str = f"{registry["base-url"]}/protocol/openid-connect/certs"
             if logger:
+                logger.debug(msg=f"Obtaining signature public key used by IAM server '{iam_server}'")
                 logger.debug(msg=f"GET {url}")
             try:
                 response: requests.Response = requests.get(url=url)
@@ -87,14 +173,31 @@ def _get_public_key(iam_server: IamServer,
                     # request succeeded
                     if logger:
                         logger.debug(msg=f"GET success, status {response.status_code}")
+                    # select the appropriate JWK
                     reply: dict[str, Any] = response.json()
-                    result = crypto_jwk_convert(jwk=reply["keys"][0],
-                                                fmt="PEM")
-                    registry["public-key"] = result
-                    lifetime: int = registry["pk-lifetime"] or 0
-                    registry["pk-expiration"] = now + lifetime
+                    jwk: dict[str, str] | None = None
+                    # replay["keys"]: list[dict[str, str]]
+                    for key in reply["keys"]:
+                        if key.get("use") == "sig":
+                            jwk = key
+                            break
+                    if jwk:
+                        # convert from 'JWK' to 'PEM' and save it for further use
+                        result = crypto_jwk_convert(jwk=jwk,
+                                                    fmt="PEM")
+                        registry["public-key"] = result
+                        lifetime: int = registry["pk-lifetime"] or 0
+                        registry["pk-expiration"] = now + lifetime if lifetime else sys.maxsize
+                        if logger:
+                            logger.debug(f"Public key obtained and saved")
+                    else:
+                        msg = "Signature public key missing from the token issuer's JWKS"
+                        if logger:
+                            logger.error(msg=msg)
+                        if isinstance(errors, list):
+                            errors.append(msg)
                 elif logger:
-                    msg: str = f"GET failure, status {response.status_code}, reason '{response.reason}'"
+                    msg: str = f"GET failure, status {response.status_code}, reason {response.reason}"
                     if hasattr(response, "content") and response.content:
                         msg += f", content {response.content}"
                     logger.error(msg=msg)
@@ -178,35 +281,6 @@ def _get_user_data(iam_server: IamServer,
     return result
 
 
-def _get_iam_server(endpoint: str,
-                    errors: list[str] | None,
-                    logger: Logger | None) -> IamServer | None:
-    """
-    Retrieve the registered *IAM* server associated with the service's invocation *endpoint*.
-
-    :param endpoint: the service's invocation endpoint
-    :param errors: incidental error messages
-    :param logger: optional logger
-    :return: the corresponding *IAM* server, or *None* if one could not be obtained
-    """
-    # declare the return variable
-    result: IamServer | None
-
-    if endpoint.startswith("jusbr"):
-        result = IamServer.IAM_JUSRBR
-    elif endpoint.startswith("keycloak"):
-        result = IamServer.IAM_KEYCLOAK
-    else:
-        result = None
-        msg: str = f"Unable to find a IAM server to service endpoint '{endpoint}'"
-        if logger:
-            logger.error(msg=msg)
-        if isinstance(errors, list):
-            errors.append(msg)
-
-    return result
-
-
 def _get_iam_registry(iam_server: IamServer,
                       errors: list[str] | None,
                       logger: Logger | None) -> dict[str, Any]:
