@@ -3,17 +3,102 @@ from flask import Request, Response, request, jsonify
 from logging import Logger
 from typing import Any
 
-from .iam_common import IamServer, _iam_lock, _get_iam_server
-from .iam_pomes import (
-    user_login, user_logout,
-    user_token, token_exchange, login_callback
+from .iam_common import (
+    IamServer, _iam_lock,
+    _get_iam_registry, _get_public_key,
+    _iam_server_from_endpoint, _iam_server_from_issuer
 )
+from .iam_actions import (
+    action_login, action_logout,
+    action_token, action_exchange, action_callback
+)
+from .token_pomes import token_get_claims, token_validate
 
 # the logger for IAM service operations
 # (used exclusively at the HTTP endpoints - all other functions receive the logger as parameter)
 __IAM_LOGGER: Logger | None = None
 
 
+def jwt_required(func: callable) -> callable:
+    """
+    Create a decorator to authenticate service endpoints with JWT tokens.
+
+    :param func: the function being decorated
+    """
+    # ruff: noqa: ANN003 - Missing type annotation for *{name}
+    def wrapper(*args, **kwargs) -> Response:
+        response: Response = __request_validate(request=request)
+        return response if response else func(*args, **kwargs)
+
+    # prevent a rogue error ("View function mapping is overwriting an existing endpoint function")
+    wrapper.__name__ = func.__name__
+
+    return wrapper
+
+
+def __request_validate(request: Request) -> Response:
+    """
+    Verify whether the HTTP *request* has the proper authorization, as per the JWT standard.
+
+    This implementation assumes that HTTP requests are handled with the *Flask* framework.
+    Because this code has a high usage frequency, only authentication failures are logged.
+
+    :param request: the *request* to be verified
+    :return: *None* if the *request* is valid, otherwise a *Response* reporting the error
+    """
+    # initialize the return variable
+    result: Response | None = None
+
+    # retrieve the authorization from the request header
+    auth_header: str = request.headers.get("Authorization")
+
+    # validate the authorization token
+    bad_token: bool = True
+    if auth_header and auth_header.startswith("Bearer "):
+        # extract and validate the JWT access token
+        token: str = auth_header.split(" ")[1]
+        claims: dict[str, Any] = token_get_claims(token=token)
+        if claims:
+            issuer: str = claims["payload"].get("iss")
+            recipient_attr: str | None = None
+            recipient_id: str = request.values.get("user-id") or request.values.get("login")
+            with _iam_lock:
+                iam_server: IamServer = _iam_server_from_issuer(issuer=issuer,
+                                                                errors=None,
+                                                                logger=__IAM_LOGGER)
+                if iam_server:
+                    # validate the token's recipient only if a user identification is provided
+                    if recipient_id:
+                        registry: dict[str, Any] = _get_iam_registry(iam_server=iam_server,
+                                                                     errors=None,
+                                                                     logger=__IAM_LOGGER)
+                        if registry:
+                            recipient_attr = registry["recipient-attr"]
+                    public_key: str = _get_public_key(iam_server=iam_server,
+                                                      errors=None,
+                                                      logger=__IAM_LOGGER)
+            # validate the token (log errors, only)
+            errors: list[str] = []
+            if public_key and token_validate(token=token,
+                                             issuer=issuer,
+                                             recipient_id=recipient_id,
+                                             recipient_attr=recipient_attr,
+                                             public_key=public_key,
+                                             errors=errors):
+                # token is valid
+                bad_token = False
+            elif __IAM_LOGGER:
+                __IAM_LOGGER.error("; ".join(errors))
+        if bad_token and __IAM_LOGGER:
+            __IAM_LOGGER.error(f"authorization refused for token {token}")
+
+    # deny the authorization
+    if bad_token:
+        result = Response(response="Authorization failed",
+                          status=401)
+    return result
+
+
 def logger_register(logger: Logger) -> None:
     """
     Register the logger for HTTP services.
@@ -33,7 +118,7 @@ def service_login() -> Response:
     Entry point for the IAM server's login service.
 
     These are the expected request parameters:
-        - user-id: optional, identifies the reference user (aliases: 'user_id', 'login')
+        - user-id: optional, identifies the reference user (alias: 'login')
         - redirect-uri: a parameter to be added to the query part of the returned URL
 
     If provided, the user identification will be validated against the authorization data
@@ -55,20 +140,20 @@ def service_login() -> Response:
     errors: list[str] = []
     with _iam_lock:
         # retrieve the IAM server
-        iam_server: IamServer = _get_iam_server(endpoint=request.endpoint,
-                                                errors=errors,
-                                                logger=__IAM_LOGGER)
+        iam_server: IamServer = _iam_server_from_endpoint(endpoint=request.endpoint,
+                                                          errors=errors,
+                                                          logger=__IAM_LOGGER)
         if iam_server:
             # obtain the login URL
-            login_url: str = user_login(iam_server=iam_server,
-                                        args=request.args,
-                                        errors=errors,
-                                        logger=__IAM_LOGGER)
+            login_url: str = action_login(iam_server=iam_server,
+                                          args=request.args,
+                                          errors=errors,
+                                          logger=__IAM_LOGGER)
             if login_url:
                 result = jsonify({"login-url": login_url})
     if errors:
-        result = Response("; ".join(errors))
-        result.status_code = 400
+        result = Response(response="; ".join(errors),
+                          status=400)
 
     # log the response
     if __IAM_LOGGER:
@@ -85,7 +170,7 @@ def service_logout() -> Response:
     """
     Entry point for the JusBR logout service.
 
-    The user is identified by the attribute *user-id*, *user_id*, or "login", provided as a request parameter.
+    The user is identified by the attribute *user-id* or "login", provided as a request parameter.
     If successful, remove all data relating to the user from the *IAM* server's registry.
     Otherwise, this operation fails silently, unless an error has ocurred.
 
@@ -101,18 +186,18 @@ def service_logout() -> Response:
     errors: list[str] = []
     with _iam_lock:
         # retrieve the IAM server
-        iam_server: IamServer = _get_iam_server(endpoint=request.endpoint,
-                                                errors=errors,
-                                                logger=__IAM_LOGGER)
+        iam_server: IamServer = _iam_server_from_endpoint(endpoint=request.endpoint,
+                                                          errors=errors,
+                                                          logger=__IAM_LOGGER)
         if iam_server:
             # logout the user
-            user_logout(iam_server=iam_server,
-                        args=request.args,
-                        errors=errors,
-                        logger=__IAM_LOGGER)
+            action_logout(iam_server=iam_server,
+                          args=request.args,
+                          errors=errors,
+                          logger=__IAM_LOGGER)
     if errors:
-        result = Response("; ".join(errors))
-        result.status_code = 400
+        result = Response(response="; ".join(errors),
+                          status=400)
     else:
         result = Response(status=204)
 
@@ -142,7 +227,7 @@ def service_callback() -> Response:
     On success, the returned *Response* will contain the following JSON:
         {
             "user-id": <reference-user-identification>,
-            "token": <token>
+            "access-token": <token>
         }
 
     :return: *Response* containing the reference user identification and the token, or *BAD REQUEST*
@@ -155,22 +240,22 @@ def service_callback() -> Response:
     token_data: tuple[str, str] | None = None
     with _iam_lock:
         # retrieve the IAM server
-        iam_server: IamServer = _get_iam_server(endpoint=request.endpoint,
-                                                errors=errors,
-                                                logger=__IAM_LOGGER)
+        iam_server: IamServer = _iam_server_from_endpoint(endpoint=request.endpoint,
+                                                          errors=errors,
+                                                          logger=__IAM_LOGGER)
         if iam_server:
             # process the callback operation
-            token_data = login_callback(iam_server=iam_server,
-                                        args=request.args,
-                                        errors=errors,
-                                        logger=__IAM_LOGGER)
+            token_data = action_callback(iam_server=iam_server,
+                                         args=request.args,
+                                         errors=errors,
+                                         logger=__IAM_LOGGER)
     result: Response
     if errors:
         result = jsonify({"errors": "; ".join(errors)})
         result.status_code = 400
     else:
         result = jsonify({"user-id": token_data[0],
-                          "token": token_data[1]})
+                          "access-token": token_data[1]})
     # log the response
     if __IAM_LOGGER:
         __IAM_LOGGER.debug(msg=f"Response {result}, {result.get_data(as_text=True)}")
@@ -186,12 +271,12 @@ def service_token() -> Response:
     """
     Entry point for retrieving a token from the *IAM* server.
 
-    The user is identified by the attribute *user-id*, *user_id*, or "login", provided as a request parameter.
+    The user is identified by the attribute *user-id* or "login", provided as a request parameter.
 
     On success, the returned *Response* will contain the following JSON:
         {
             "user-id": <reference-user-identification>,
-            "token": <token>
+            "access-token": <token>
         }
 
     :return: *Response* containing the user reference identification and the token, or *BAD REQUEST*
@@ -202,23 +287,23 @@ def service_token() -> Response:
 
     # obtain the user's identification
     args: dict[str, Any] = request.args
-    user_id: str = args.get("user-id") or args.get("user_id") or args.get("login")
+    user_id: str = args.get("user-id") or args.get("login")
 
     errors: list[str] = []
     token: str | None = None
     if user_id:
         with _iam_lock:
             # retrieve the IAM server
-            iam_server: IamServer = _get_iam_server(endpoint=request.endpoint,
-                                                    errors=errors,
-                                                    logger=__IAM_LOGGER)
+            iam_server: IamServer = _iam_server_from_endpoint(endpoint=request.endpoint,
+                                                              errors=errors,
+                                                              logger=__IAM_LOGGER)
             if iam_server:
                 # retrieve the token
                 errors: list[str] = []
-                token: str = user_token(iam_server=iam_server,
-                                        args=args,
-                                        errors=errors,
-                                        logger=__IAM_LOGGER)
+                token: str = action_token(iam_server=iam_server,
+                                          args=args,
+                                          errors=errors,
+                                          logger=__IAM_LOGGER)
     else:
         msg: str = "User identification not provided"
         errors.append(msg)
@@ -227,11 +312,11 @@ def service_token() -> Response:
 
     result: Response
     if errors:
-        result = Response("; ".join(errors))
-        result.status_code = 400
+        result = Response(response="; ".join(errors),
+                          status=400)
     else:
         result = jsonify({"user-id": user_id,
-                          "token": token})
+                          "access-token": token})
     # log the response
     if __IAM_LOGGER:
         __IAM_LOGGER.debug(msg=f"Response {result}, {result.get_data(as_text=True)}")
@@ -247,8 +332,8 @@ def service_exchange() -> Response:
 
     This is currently limited to the *KEYCLOAK* server. The token itself is stored in *KEYCLOAK*'s registry.
     The expected request parameters are:
-        - user-id: identification for the reference user (aliases: 'user_id', 'login')
-        - token: the token to be exchanged
+        - user-id: identification for the reference user (alias: 'login')
+        - access-token: the token to be exchanged
 
     If the exchange is successful, the token data is stored in the *IAM* server's registry, and returned.
     Otherwise, *errors* will contain the appropriate error message.
@@ -271,21 +356,21 @@ def service_exchange() -> Response:
     errors: list[str] = []
     with _iam_lock:
         # retrieve the IAM server (currently, only 'IAM_KEYCLOAK' is supported)
-        iam_server: IamServer = _get_iam_server(endpoint=request.endpoint,
-                                                errors=errors,
-                                                logger=__IAM_LOGGER)
+        iam_server: IamServer = _iam_server_from_endpoint(endpoint=request.endpoint,
+                                                          errors=errors,
+                                                          logger=__IAM_LOGGER)
         # exchange the token
         token_data: dict[str, Any] | None = None
         if iam_server:
             errors: list[str] = []
-            token_data = token_exchange(iam_server=iam_server,
-                                        args=request.args,
-                                        errors=errors,
-                                        logger=__IAM_LOGGER)
+            token_data = action_exchange(iam_server=iam_server,
+                                         args=request.args,
+                                         errors=errors,
+                                         logger=__IAM_LOGGER)
     result: Response
     if errors:
-        result = Response("; ".join(errors))
-        result.status_code = 400
+        result = Response(response="; ".join(errors),
+                          status=400)
     else:
         result = jsonify(token_data)
 
