@@ -7,7 +7,7 @@ from pypomes_core import (
 from typing import Any, Final
 
 from .iam_common import _IAM_SERVERS, IamServer, _iam_lock
-from .iam_pomes import user_token
+from .iam_actions import action_token
 
 JUSBR_CLIENT_ID: Final[str] = env_get_str(key=f"{APP_PREFIX}_JUSBR_CLIENT_ID")
 JUSBR_CLIENT_SECRET: Final[str] = env_get_str(key=f"{APP_PREFIX}_JUSBR_CLIENT_SECRET")
@@ -24,6 +24,7 @@ JUSBR_ENDPOINT_TOKEN: Final[str] = env_get_str(key=f"{APP_PREFIX}_JUSBR_ENDPOINT
 
 JUSBR_PUBLIC_KEY_LIFETIME: Final[int] = env_get_int(key=f"{APP_PREFIX}_JUSBR_PUBLIC_KEY_LIFETIME",
                                                     def_value=86400)  # 24 hours
+JUSBR_REALM: Final[str] = env_get_str(key=f"{APP_PREFIX}_JUSBR_REALM")
 JUSBR_RECIPIENT_ATTR: Final[str] = env_get_str(key=f"{APP_PREFIX}_JUSBR_RECIPIENT_ATTR",
                                                def_value="preferred_username")
 JUSBR_URL_AUTH_BASE: Final[str] = env_get_str(key=f"{APP_PREFIX}_JUSBR_URL_AUTH_BASE")
@@ -31,6 +32,7 @@ JUSBR_URL_AUTH_BASE: Final[str] = env_get_str(key=f"{APP_PREFIX}_JUSBR_URL_AUTH_
 
 def jusbr_setup(flask_app: Flask,
                 base_url: str = JUSBR_URL_AUTH_BASE,
+                realm: str = JUSBR_REALM,
                 client_id: str = JUSBR_CLIENT_ID,
                 client_secret: str = JUSBR_CLIENT_SECRET,
                 client_timeout: int = JUSBR_CLIENT_TIMEOUT,
@@ -47,6 +49,7 @@ def jusbr_setup(flask_app: Flask,
 
     :param flask_app: the Flask application
     :param base_url: base URL to request JusBR services
+    :param realm: the JusBR realm
     :param client_id: the client's identification with JusBR
     :param client_secret: the client's password with JusBR
     :param client_timeout: timeout for login authentication (in seconds,defaults to no timeout)
@@ -64,7 +67,7 @@ def jusbr_setup(flask_app: Flask,
     cache["users"] = {}
     with _iam_lock:
         _IAM_SERVERS[IamServer.IAM_JUSRBR] = {
-            "base-url": base_url,
+            "base-url": f"{base_url}/realms/{realm}",
             "client-id": client_id,
             "client-secret": client_secret,
             "client-timeout": client_timeout,
@@ -115,8 +118,8 @@ def jusbr_get_token(user_id: str,
     # retrieve the token
     args: dict[str, Any] = {"user-id": user_id}
     with _iam_lock:
-        result = user_token(iam_server=IamServer.IAM_JUSRBR,
-                            args=args,
-                            errors=errors,
-                            logger=logger)
+        result = action_token(iam_server=IamServer.IAM_JUSRBR,
+                              args=args,
+                              errors=errors,
+                              logger=logger)
     return result
