@@ -7,7 +7,7 @@ from pypomes_core import (
 from typing import Any, Final
 
 from .iam_common import _IAM_SERVERS, IamServer, _iam_lock
-from .iam_pomes import user_token
+from .iam_actions import action_token
 
 KEYCLOAK_CLIENT_ID: Final[str] = env_get_str(key=f"{APP_PREFIX}_KEYCLOAK_CLIENT_ID")
 KEYCLOAK_CLIENT_SECRET: Final[str] = env_get_str(key=f"{APP_PREFIX}_KEYCLOAK_CLIENT_SECRET")
@@ -129,8 +129,8 @@ def keycloak_get_token(user_id: str,
     # retrieve the token
     args: dict[str, Any] = {"user-id": user_id}
     with _iam_lock:
-        result = user_token(iam_server=IamServer.IAM_KEYCLOAK,
-                            args=args,
-                            errors=errors,
-                            logger=logger)
+        result = action_token(iam_server=IamServer.IAM_KEYCLOAK,
+                              args=args,
+                              errors=errors,
+                              logger=logger)
     return result
